% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/NDepositsPmf.R
\name{NDepositsPmf}
\alias{NDepositsPmf}
\title{Construct the pmf for the number of undiscovered deposits}
\usage{
NDepositsPmf(type, pmf.args, description = "")
}
\arguments{
\item{type}{Character string with the type of pmf (See Details).}

\item{pmf.args}{List with the arguments for the specified type (See Details).}

\item{description}{Character string with a short description of the pmf.}
}
\value{
If the input arguments have an error, the R-value NULL is returned.
Otherwise, a list with the following components is returned.

\item{Type}{Input argument type}

\item{pmf.args}{Input argument pmf.args}

\item{description}{Input argument description}

\item{call}{Function call}

\item{probs}{Vector containing just the non-zero probabilities
in the pmf.}

\item{nDeposits}{Vector containing the number of undiscovered
deposits that are associated with the non-zero probabilities in the pmf.
The size of vector nDeposits is equal to the size of vector probs.}

\item{theMean}{The expected value (mean) of the number of
undiscovered deposits within the permissive tract.}

\item{theVar}{The variance of the number of undiscovered deposits
within the permissive tract.}

\item{entropy}{The information entropy, which is calculated with
the natural logarithm.}
}
\description{
Construct the probability mass function (pmf) for the number of
undiscovered deposits within the permissive tract
}
\details{
The type must be either "NegBinomial" or "Custom". Each type, as well as
it associated arguments, is described below.

\emph{NegBinomial}

List pmf.args has one required element and four optional arguments.
The required element \code{nDepEst} is a dataframe containing
the estimates of the numbers of undiscovered deposits by the members of the
assessment team. Each row of the dataframe has information for one member.
There are five columns in the dataframe.
Column 1 ("Name") lists the member identifiers.
Column 2 ("Weight") lists the weights that multiply the cost function.
Column 3 ("N90") lists the estimated number of the undiscovered deposits,
n90, for which P( N >= n90 ) = 0.9 where N is the random variable representing
the number of undiscovered deposits.
Likewise, column 4 ("N50") lists n50 for which P( N >= n50 ) = 0.5, and
column 5 ("N10") lists n10 for which P( N >= n10 ) = 0.1. Within each row
of the dataframe, n90, n50, and n10 must be in strickly ascending order. That
is, n90 <= n50 <= n10.

There are four optional arguments that may be added to the list of parameters
for the pmf: \code{nGridPoints}, \code{power},
\code{isOptimized}, and \code{probRightTail}.
To understand the first three arguments,
it is necessary to understand how the two
parameters that are needed to characterize a negative binomial pmf are
estimated. The two parameters are specified on a two-dimensional grid, and
\code{nGridPoints} is the number of grid points in each dimension of the
grid. At
each grid point, a cost function is calculated, and a parameter in that
cost function is \code{power}.
If the value of argument \code{isOptimized} is FALSE, then
the two parameter values associated with the grid point with the lowest cost
are used as the parameters for negative binomial pmf. Otherwise, these two
parameter values are the starting point for an optimization that finds their
optimal values.

To understand the fourth optional argument, it is necessary to understand
the negative binomial pmf itself. The random variable that represents the
number of undiscovered deposits extends from 0 to infinity. Of course,
the probability calculations cannot be performed for an infinite range.
At some point in the right tail, the probabilities are
so small that they have no practical effect on the probability calculations.
Thus, the pmf is truncated at an appropriate large value. This upper
truncation point is the quantile above which the sum of the probabilities
(from that quantile to infinity)
is less than (or equal to) \code{probRightTail}.
The lower truncation point is that quantile for which the probability is
less than or equal to that at the upper truncation point. After truncation,
the pmf is normalized so that the sum of its probabilities is 1.

The default values for these four optional arguments are appropriate for
most assessments, so they should rarely be changed.

The computation of the negative binomial pmf requires
a long time---but usually less than a minute.

\emph{Custom}

List pmf.args has two elements, which are named "nDeposits" and
"relProbabilities". nDeposits is an integer vector, and relProbabilities is
a real vector.

Regarding vector nDeposits, all elements must be in strictly ascending order.
That is,
0 <= nDeposits[1] < nDeposits[2] < nDeposits[3] and so on. Regarding
vector relProbabilities, the elements are postive,
real-valued numbers. The sum of the elements in relProbabilities is
not required to be 1. Vector nDeposits
and vector relProbabilites must have the same length.
}
\examples{
pmf1 <- NDepositsPmf("NegBinomial", list(nDepEst = ExampleDepEst1))
plot(pmf1)

pmf2 <- NDepositsPmf("NegBinomial",
         list(nDepEst = ExampleDepEst1, probRightTail = 0.0005,
         nGridPoints = 65, power = 1.0, isOptimized = FALSE))
plot(pmf2)

pmf3 <- NDepositsPmf("Custom", list(nDeposits = 1, relProbabilities = 1))
plot(pmf3)

pmf4 <- NDepositsPmf("Custom",
         list(nDeposits = c(2, 3), relProbabilities = c(0.3, 0.7)))
plot(pmf4)

}
\references{
Ellefsen, K.J., Phillips, J.D.,
2017, Probability calculations for three-part mineral resource assessments,
U.S. Geological Survey Report Techniques and Methods XXXX, XX p.
}

